/* This dialog box allows the user to specify options for CSG objects. */

/* Copyright (C) 2001-2002 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY 
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion;

import artofillusion.*;
import artofillusion.object.*;
import artofillusion.object.TriangleMesh.*;
import artofillusion.ui.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class CSGDialog extends Dialog implements ActionListener, ItemListener
{
  CSGObject theObject;
  CSGModeller modeller;
  Choice opChoice;
  Button okButton, cancelButton;
  ObjectPreviewCanvas preview;
  int operation[];
  boolean ok;
  
  private static int counter = 1;

  public CSGDialog(EditingWindow window, CSGObject obj)
  {
    super((Frame) window, "Boolean", true);
    theObject = obj;
    Scene scene = window.getScene();
    
    // Layout the window.
    
    setLayout(new GridBagLayout());
    GridBagConstraints gc = new GridBagConstraints();
    gc.anchor = gc.WEST;
    gc.gridx = 0;
    add(new Label("Operation:"), gc);
    add(opChoice = new Choice(), gc);
    int i = 0;
    operation = new int [5];
    if (obj.getObject1().object.isClosed() && obj.getObject2().object.isClosed())
      {
        opChoice.add("Union");
        operation[i++] = CSGObject.UNION;
      }
    opChoice.add("Intersection");
    operation[i++] = CSGObject.INTERSECTION;
    if (obj.getObject2().object.isClosed())
      {
        opChoice.add("First - Second");
        operation[i++] = CSGObject.DIFFERENCE12;
      }
    if (obj.getObject1().object.isClosed())
      {
        opChoice.add("Second - First");
        operation[i++] = CSGObject.DIFFERENCE21;
      }
    for (int j = 0; j < i; j++)
      if (obj.getOperation() == operation[j])
        opChoice.select(j);
    opChoice.addItemListener(this);

    // Add the preview canvas.
    
    gc.gridx = 0;
    gc.fill = gc.BOTH;
    gc.weightx = gc.weighty = 1.0;
    add(preview = new ObjectPreviewCanvas(null), gc);
    preview.setPreferredSize(200, 200);
    
    // Add the buttons at the bottom.
    
    gc.gridwidth = 2;
    gc.weightx = gc.weighty = 0.0;
    Panel p = new Panel();
    p.add(okButton = Translate.button("ok", this));
    p.add(cancelButton = Translate.button("cancel", this));
    add(p, gc);
    makePreview();
    pack();
    show();
  }
  
  // Deal with changes to the checkboxes and choices.
  
  public void itemStateChanged(ItemEvent e)
  {
    makePreview();
  }
  
  // Deal with clicks on the buttons.
  
  public void actionPerformed(ActionEvent e)
  {
    if (okButton == e.getSource())
      {
	theObject.setOperation(operation[opChoice.getSelectedIndex()]);
	ok = true;
      }
    dispose();
  }
  
  // Create a preview object.
  
  private void makePreview()
  {
    if (modeller == null)
      {
        double tol = ModellingApp.getPreferences().getInteractiveSurfaceError();
        TriangleMesh mesh1, mesh2;
    
        mesh1 = theObject.getObject1().object.convertToTriangleMesh(tol);
        mesh2 = theObject.getObject2().object.convertToTriangleMesh(tol);
        modeller = new CSGModeller(mesh1, mesh2, theObject.getObject1().coords, theObject.getObject2().coords);
      }
    preview.setObject(modeller.getMesh(operation[opChoice.getSelectedIndex()]));
    preview.updateImage();
    preview.repaint();
  }
  
  // Determine whether the user clicked the OK button.
  
  public boolean clickedOk()
  {
    return ok;
  }
}