/* A custom class loader for getting classes from a jar or zip file. */
 
/* Copyright (C) 1999-2001 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY 
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion;

import java.io.*;
import java.util.Hashtable;
import java.util.zip.*;

class JarClassLoader extends ClassLoader
{
  ZipFile zf;
  Hashtable cache = new Hashtable();

  public JarClassLoader(ZipFile zf)
  {
    super();
    this.zf = zf;
  }

  public synchronized Class loadClass(String name, boolean resolve) throws ClassNotFoundException
  {
    Class c;

    // First see if it's already in the hashtable.

    c = (Class) cache.get(name);
    if (c == null)
      {
	// See if it's a system class.

	try
	  {
	    return findSystemClass(name);
	  }
	catch (ClassNotFoundException e)
	  {
	  }

	// Look for it in the zip file.

	ZipEntry ze = zf.getEntry(name.replace('.', '/')+".class");
	if (ze == null)
	  throw new ClassNotFoundException(name);
	int size = (int) ze.getSize();
	byte data[] = new byte [size];

	try
	  {
	    InputStream in = new BufferedInputStream(zf.getInputStream(ze));
	    for (int i = 0; i < size; i++)
	      data[i] = (byte) in.read();
	    in.close();
	  }
	catch (IOException e)
	  {
	    throw new ClassNotFoundException(name);
	  }
	c = defineClass(name, data, 0, data.length);
	cache.put(name, c);
      }
    if (resolve)
      resolveClass(c);
    return c;
  }
}
