/* Copyright (C) 1999-2002 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */


package artofillusion;

/** Represents the state of an object as needed for undo management.
 *
 * @author  Michael Butscher
 * @see ObjectStateProducer
 */
public interface ObjectState
{
  /** Set creator of this state. */
  public void setStateCreator(ObjectStateCreator c);

  /** Get creator of this state. */
  public ObjectStateCreator getStateCreator();

  /** Apply the state to its StateCreator and return a state which can undo
    these changes or null if this is not possible. The state does not need
    to be valid anymore after calling this method, so it may return itself.*/
  public ObjectState replay();
}
