/* This is the window for editing application-wide preferences. */

/* Copyright (C) 2002 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion;

import artofillusion.ui.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class PreferencesWindow
{
  public PreferencesWindow(Frame parent)
  {
    ApplicationPreferences prefs = ModellingApp.getPreferences();
    Choice defaultRendChoice = getRendererChoice(prefs.getDefaultRenderer());
    Choice objectRendChoice = getRendererChoice(prefs.getObjectPreviewRenderer());
    Choice texRendChoice = getRendererChoice(prefs.getTexturePreviewRenderer());
    Choice displayChoice = new Choice();
    Choice localeChoice = new Choice();
    ValueField interactiveTolField = new ValueField(prefs.getInteractiveSurfaceError(), ValueField.POSITIVE);
    Checkbox objectSceneAutosyncCheckbox = new Checkbox("", prefs.getObjectSceneAutosync());

    displayChoice.add(Translate.text("menu.wireframeDisplay"));
    displayChoice.add(Translate.text("menu.shadedDisplay"));
    displayChoice.add(Translate.text("menu.smoothDisplay"));
    displayChoice.select(prefs.getDefaultDisplayMode());
    Locale languages[] = Translate.getAvailableLocales();
    for (int i = 0; i < languages.length; i++)
      {
        localeChoice.add(languages[i].getDisplayName(prefs.getLocale()));
        if (prefs.getLocale().equals(languages[i]))
          localeChoice.select(i);
      }
    ComponentsDialog dlg = new ComponentsDialog(parent, "Art of Illusion Preferences",
      new Component [] {defaultRendChoice, objectRendChoice, texRendChoice,
      displayChoice, interactiveTolField, localeChoice, objectSceneAutosyncCheckbox},
      new String [] {"Default Renderer",
      "Object Preview Renderer", "Texture Preview Renderer", "Default Display Mode",
      "Interactive Surface Error", "Language", "Sync object editors with scene"});
    if (!dlg.clickedOk())
      return;
    Renderer renderers[] = ModellingApp.getRenderers();
    if (renderers.length > 0)
      {
        prefs.setDefaultRenderer(renderers[defaultRendChoice.getSelectedIndex()]);
        prefs.setObjectPreviewRenderer(renderers[objectRendChoice.getSelectedIndex()]);
        prefs.setTexturePreviewRenderer(renderers[texRendChoice.getSelectedIndex()]);
      }
    prefs.setDefaultDisplayMode(displayChoice.getSelectedIndex());
    prefs.setInteractiveSurfaceError(interactiveTolField.getValue());
    if (!prefs.getLocale().equals(languages[localeChoice.getSelectedIndex()]))
      new MessageDialog(parent, "The new language setting will apply to new windows, but will not affect any windows which are currently open.");
    prefs.setLocale(languages[localeChoice.getSelectedIndex()]);
    prefs.setObjectSceneAutosync(objectSceneAutosyncCheckbox.getState());
    prefs.savePreferences();
  }

  /** Create a Choice for selecting a renderer. */

  private Choice getRendererChoice(Renderer selected)
  {
    Renderer renderers[] = ModellingApp.getRenderers();
    Choice c = new Choice();

    for (int i = 0; i < renderers.length; i++)
      c.add(renderers[i].getName());
    if (selected != null)
      c.select(selected.getName());
    return c;
  }
}