/* The Renderer interface defines the methods for rendering a scene. */

/* Copyright (C) 1999,2000,2003 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY 
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion;

import artofillusion.object.*;
import java.awt.*;

public interface Renderer
{
  /** Get the name of the renderer.*/

  public String getName();

  /** Begin rendering a scene.  If depthOfField is set to 0, then depth of field effect 
      will not be used.  Some renderers may not support this effect, in which case  
      depthOfField and focalDist will be ignored. */

  public void renderScene(Scene theScene, Camera theCamera, RenderListener listener, SceneCamera sceneCamera);

  /** Cancel a rendering which is in progress. */
  
  public void cancelRendering(Scene theScene);

  /** Get a Panel in which the user can specify options about how the scene should be rendered. */ 

  public Panel getConfigPanel();

  /** Record the values which the user has entered into the configuration panel.  If all
      values are valid, return true.  Otherwise, return false. */

  public boolean recordConfiguration();

  /** Configure the renderer in a way which is appropriate for rendering previews.  This should
      try to find a reasonable balance between speed and image quality. */

  public void configurePreview();
}