/* ThickenMeshTool is an EditingTool used for making pieces of TriangleMeshes thicker
   or thinner. */

/* Copyright (C) 2001-2002 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY 
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion;

import artofillusion.math.*;
import artofillusion.object.*;
import artofillusion.ui.*;
import artofillusion.object.Mesh.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.Vector;

public class ThickenMeshTool extends EditingTool
{
  boolean dragged;
  static Image icon, selectedIcon;
  Point clickPoint;
  Vec3 norm[];
    
  public ThickenMeshTool(EditingWindow fr)
  {
    super(fr);
    icon = loadImage("outsetPoints.gif");
    selectedIcon = loadImage("selected/outsetPoints.gif");
  }

  public void activate()
  {
    super.activate();

    MeshViewer view = ((MeshEditorWindow) theFrame).getCurrentView();
    int selected[] = view.getSelectionDistance();
    for (int i = 0; i < selected.length; i++)
      if (selected[i] == 0)
	{
	  theWindow.setHelpText(Translate.text("thickenMeshTool.helpText"));
	  return;
	}
    theWindow.setHelpText(Translate.text("thickenMeshTool.errorText"));
  }

  public Image getIcon()
  {
    return icon;
  }

  public Image getSelectedIcon()
  {
    return selectedIcon;
  }

  public void mousePressed(MouseEvent e, ViewerCanvas view)
  {
    Mesh mesh = (Mesh) ((MeshViewer) view).getObject().object;
    MeshVertex v[] = mesh.getVertices();
    
    clickPoint = e.getPoint();
    dragged = false;
    norm = mesh.getNormals();
  }
  
  public void mouseDragged(MouseEvent e, ViewerCanvas view)
  {
    Mesh mesh = (Mesh) ((MeshViewer) view).getObject().object;
    MeshVertex vert[] = mesh.getVertices();
    Graphics g = view.getGraphics();
    Camera cam = view.getCamera();
    Point dragPoint = e.getPoint();
    int i, size = MeshViewer.HANDLE_SIZE, selectDist[] = ((MeshViewer) view).getSelectionDistance();
    double distance;
    Vec3 v[];
    Vec2 p;

    if (dragged)
      view.drawImage(g);
    dragged = true;
    distance = 0.01*(clickPoint.y - dragPoint.y);
    v = findNewPositions(mesh, vert, (MeshViewer) view, distance);
    g.setColor(Color.gray);
    for (i = 0; i < v.length; i++)
      if (selectDist[i] > -1)
	{
	  p = cam.getObjectToScreen().timesXY(v[i]);
	  g.fillRect(((int) p.x) - size/2, ((int) p.y) - size/2, size, size);
	}
    ((MeshViewer) view).drawDraggedSelection(g, cam, v, true);
    NumberFormat format = NumberFormat.getNumberInstance();
    format.setMaximumFractionDigits(2);
    if (distance < 0.0)
      theWindow.setHelpText(Translate.text("thickenMeshTool.dragText.inward", format.format(-distance)));
    else
      theWindow.setHelpText(Translate.text("thickenMeshTool.dragText.outward", format.format(distance)));
    g.dispose();
  }

  public void mouseReleased(MouseEvent e, ViewerCanvas view)
  {
    Object3D meshobj = ((MeshViewer) view).getObject().object;
    Mesh mesh = (Mesh) meshobj;
    MeshVertex vert[] = mesh.getVertices();
    Camera cam = view.getCamera();
    Point dragPoint = e.getPoint();
    int i;
    double distance;
    Vec3 v[];

    distance = 0.01*(clickPoint.y - dragPoint.y);
    if (distance != 0.0)
      {
	theWindow.setUndoRecord(new UndoRecord(theWindow, false, UndoRecord.COPY_OBJECT, new Object [] {mesh, mesh.duplicate()}));
	v = findNewPositions(mesh, vert, (MeshViewer) view, distance);
	mesh.setVertices(v);
      }
    meshobj.informChanged(this, ModelEvent.CHANGEDUR_OBJECTEDITOR);
    theWindow.setHelpText(Translate.text("thickenMeshTool.helpText"));
    norm = null;
  }
  
  /* Find the new positions of the vertices . */

  Vec3 [] findNewPositions(Mesh mesh, MeshVertex vert[], MeshViewer view, double distance)
  {
    Vec3 v[] = new Vec3 [vert.length];
    int selected[] = view.getSelectionDistance();
    
    for (int i = 0; i < v.length; i++)
      {
	if (selected[i] == 0)
	  v[i] = norm[i].times(distance);
	else
	  v[i] = new Vec3();
      }
    ((MeshEditorWindow) theFrame).adjustDeltas(v);
    for (int i = 0; i < vert.length; i++)
      v[i].add(vert[i].r);
    return v;
  }
}