
/* Copyright (C) 1999-2002 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion;

import artofillusion.ui.*;
import java.awt.*;
import java.awt.event.*;

/** TitleWindow displays a window containing the title and credits.  It is actually a modal 
    dialog, but it displays the dialog from a new thread, so that the calling thread is not 
    blocked.  Clicking anywhere in the dialog dismisses it. */

public class TitleWindow extends Window implements MouseListener
{
  boolean closed;

  public TitleWindow()
  {
    super(new Frame());
    GridBagConstraints gc = new GridBagConstraints();
    Font font1 = new Font("Serif", Font.ITALIC, 36);
    Font font2 = new Font("Serif", Font.PLAIN, 12);
    Label l;

    add(new ImageCanvas("/artofillusion/titleImage.jpg"), BorderLayout.NORTH);
    BorderPanel p = new BorderPanel();
    p.setBackground(Color.white);
    add(p, BorderLayout.CENTER);
    p.setLayout(new GridBagLayout());
    gc.gridx = 0;
    gc.insets.top = 5;
    p.add(l = new Label("Art of Illusion v"+ModellingApp.VERSION, Label.CENTER), gc);
    l.setFont(font2);
    gc.insets.top = 0;
    p.add(l = new Label("Copyright 1999-2003 by Peter Eastman and others", Label.CENTER), gc);
    l.setFont(font2);
    p.add(l = new Label("(See the README file for details.)", Label.CENTER), gc);
    l.setFont(font2);
    p.add(l = new Label("This program may be freely distributed under", Label.CENTER), gc);
    l.setFont(font2);
    gc.insets.bottom = 5;
    p.add(l = new Label("the terms of the accompanying license.", Label.CENTER), gc);
    l.setFont(font2);
    pack();
    recursivelyAddListeners(this);
    ModellingApp.centerWindow(this);
    show();
  }

  /** Close the window. */

  public void dispose()
  {
    if (closed)
      return;
    closed = true;
    super.dispose();
    ((Frame) getParent()).dispose();
  }

  /** Add this as a MouseListener to every Component in the Window. */

  private void recursivelyAddListeners(Component c)
  {
    Component children[];

    c.addMouseListener(this);
    if (c instanceof Container)
      {
	children = ((Container) c).getComponents();
	for (int i = 0; i < children.length; i++)
	  recursivelyAddListeners(children[i]);
      }
  }

  public void mouseClicked(MouseEvent e)
  {
    dispose();
  }

  public void mousePressed(MouseEvent e)
  {
  }

  public void mouseReleased(MouseEvent e)
  {
  }

  public void mouseEntered(MouseEvent e)
  {
  }

  public void mouseExited(MouseEvent e)
  {
  }
}