/* Copyright (C) 1999-2003 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY 
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion;

import artofillusion.ui.*;
import java.awt.*;
import java.awt.event.*;

/** This class implements the dialog box which is used for the "Transform Points" command.
    It allows the user enter values by which to translate, rotate, and scale the selected
    Points.  It also allows them to specify whether the rotation and scaling should be done
    with respect to the origin of object coordinates, or the center of the selected points. */

public class TransformPointsDialog extends Dialog implements ActionListener, FocusListener
{
  double finalValues[];
  TextField fields[];
  Choice centerChoice;
  boolean clicked;

  public TransformPointsDialog(Frame parent)
  {
    super(parent, "Transform Points", true);
    fields = new TextField [9];
    layoutDialog();
    pack();
    setResizable(false);
    ModellingApp.centerWindow(this);
    fields[0].requestFocus();
    show();
  }

  public double [] getValues()
  {
    return finalValues;
  }
  
  public boolean useSelectionCenter()
  {
    return (centerChoice.getSelectedIndex() == 0);
  }
  
  void layoutDialog()
  {
    int i;
    Panel p = new Panel();
    GridBagConstraints c = new GridBagConstraints();

    setLayout(new GridBagLayout());
    c.anchor = GridBagConstraints.EAST;
    c.ipadx = 5;
    c.gridx = 0;
    c.gridy = 1;
    add(Translate.label("Move"), c);
    c.gridy = GridBagConstraints.RELATIVE;
    add(Translate.label("Rotate"), c);
    add(Translate.label("Scale"), c);
    c.anchor = GridBagConstraints.CENTER;
    c.ipadx = 0;
    c.gridx = 1;
    c.gridy = 0;
    add(new Label("        X        "), c);
    c.gridx = GridBagConstraints.RELATIVE;
    add(new Label("        Y        "), c);
    add(new Label("        Z        "), c);
    c.fill = GridBagConstraints.HORIZONTAL;
    c.gridy = 1;
    c.weightx = 1.0;
    add(fields[0] = new TextField(""), c);
    add(fields[1] = new TextField(""), c);
    add(fields[2] = new TextField(""), c);
    c.gridy = 2;
    add(fields[3] = new TextField(""), c);
    add(fields[4] = new TextField(""), c);
    add(fields[5] = new TextField(""), c);
    c.gridy = 3;
    add(fields[6] = new TextField(""), c);
    add(fields[7] = new TextField(""), c);
    add(fields[8] = new TextField(""), c);
    c.gridx = 0;
    c.gridy = 4;
    c.gridwidth = 5;
    add(p, c);
    p.add(new Label("Transform Around"));
    p.add(centerChoice = new Choice());
    centerChoice.add("Selection Center");
    centerChoice.add("Object Origin");
    c.gridy = 5;
    add(p = new Panel(), c);
    p.add(Translate.button("ok", this));
    p.add(Translate.button("cancel", this));
    MouseListener listen = new MouseAdapter() {
      public void mousePressed(MouseEvent ev)
      {
        clicked = true;
      }
    };
    for (i = 0; i < 9; i++)
      {
	fields[i].addActionListener(this);
	fields[i].addFocusListener(this);
	fields[i].addMouseListener(listen);
      }
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e)
      {
        dispose();
      }
    } );
  }

  public void actionPerformed(ActionEvent e)
  {
    String command = e.getActionCommand();
    String finalStrings[] = new String [9];
    int i;

    if (!command.equals("cancel"))
      {
	finalValues = new double [9];
	for (i = 0; i < 9; i++)
	  {
	    try
	      {
		finalValues[i] = Double.valueOf(fields[i].getText()).doubleValue();
	      }
	    catch (NumberFormatException ex)
	      {
		if (i < 6)
		  finalValues[i] = 0.0;
		else
		  finalValues[i] = 1.0;
	      }
	  }
      }
    dispose();
  }

  public void focusGained(FocusEvent e)
  {
    TextField t = (TextField) e.getSource();

    if (!clicked)
      {
	t.setSelectionStart(0);
	t.setSelectionEnd(t.getText().length());
      }
    clicked = false;
  }

  public void focusLost(FocusEvent e)
  {
    TextField t = (TextField) e.getSource();

    t.setSelectionStart(0);
    t.setSelectionEnd(0);
  }
}