/* Copyright (C) 1999-2000 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion;

import artofillusion.animation.*;
import artofillusion.math.*;
import artofillusion.object.*;
import artofillusion.ui.*;

/** Class to handle undo and redo operations. Do not mix up with
  javax.swing.undo.UndoManager!*/

public class UndoManager  // ??? implements ModelListener
{
  StateStack undo, redo;
  
  static UndoManager defaultManager = null;

  public UndoManager()
  {
    undo = new StateStack();
    redo = new StateStack();
  }

  public StateStack getUndoStack()
  {
    return undo;
  }

  public StateStack getRedoStack()
  {
    return redo;
  }

  public void clearUndo()
  {
    undo.clear();
  }

  public void clearRedo()
  {
    redo.clear();
  }

  /** Pushes a state on the undo stack and clears redo stack automatically. */
  public void pushUndo(ObjectState state)
  {
    undo.push(state, this);
    clearRedo();
  }
  
  /** #StateStack.replay top of from stack and put new state on to stack.
      Helper function for #commandUndo, #commandRedo. */
  protected void shiftState(StateStack from, StateStack to)
  {
    if (from.isEmpty())
      return;
    
    ObjectState newstate = from.pop(this).replay();
    
    if (newstate != null)
      to.push(newstate, this);
  }

  /** Called if the user selected the undo menu item. Takes the state on top
    of the undo stack, replays it and pushes the returned redo state on the
    redo stack.*/
  
  public void commandUndo()
  {
    shiftState(undo, redo);
  }

   /** Called if the user selected the redo menu item. Takes the state on top
     of the redo stack, replays it and pushes the returned undo state on the
     undo stack.*/
  
  public void commandRedo()
  {
    shiftState(redo, undo);
  }

  public boolean canUndo()
  {
    return !undo.isEmpty();
  }

  public boolean canRedo()
  {
    return !redo.isEmpty();
  }
  
  public static UndoManager getDefaultManager()
  {
    if (defaultManager == null)
      defaultManager = new UndoManager();

    return defaultManager;
  }
}