/* This class creates a window for displaying output from scripts. */

/* Copyright (C) 2002 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY 
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion.script;

import artofillusion.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class ScriptOutputWindow extends OutputStream implements ActionListener
{
  Frame window;
  TextArea text;
  
  public void write(int b)
  {
    if (window == null)
      createWindow();
    else if (!window.isVisible())
      window.show();
    text.append(String.valueOf((char) b));
  }
  
  public void write(byte b[], int off, int len)
  {
    if (window == null)
      createWindow();
    else if (!window.isVisible())
      window.show();
    text.append(new String(b, off, len));
  }

  /** Create the window. */
  
  private void createWindow()
  {
    window = new Frame("Script Output");
    text = new TextArea(10, 60);
    text.setFont(ModellingApp.defaultFont);
    window.add(text, BorderLayout.CENTER);
    Button b;
    Panel p = new Panel();
    p.add(b = new Button("Close"));
    b.addActionListener(this);
    window.add(p, BorderLayout.SOUTH);
    window.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e)
      {
        closeWindow();
      }
    } );
    window.pack();
    window.show();
  }
  
  /** Hide the window. */
  
  private void closeWindow()
  {
    window.setVisible(false);
    text.setText("");
  }
  
  public void actionPerformed(ActionEvent ev)
  {
    closeWindow();
  }
}
