/* UniformMapping is the TextureMapping for UniformTextures. */

/* Copyright (C) 1999-2002 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY 
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion.texture;

import artofillusion.*;
import artofillusion.math.*;
import artofillusion.object.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class UniformMapping extends TextureMapping
{
  UniformTexture texture;
  
  public UniformMapping(Texture theTexture)
  {
    texture = (UniformTexture) theTexture;
  }

  public Texture getTexture()
  {
    return texture;
  }

  public RenderingTriangle mapTriangle(int v1, int v2, int v3, int n1, int n2, int n3, Vec3 vert[])
  {
    return new UniformTriangle(v1, v2, v3, n1, n2, n3, this);
  }
  
  public void getTextureSpec(Vec3 pos, TextureSpec spec, double angle, double size, double t, double param[])
  {
    if (!appliesToFace(angle > 0.0))
      {
        spec.diffuse.setRGB(0.0f, 0.0f, 0.0f);
        spec.specular.setRGB(0.0f, 0.0f, 0.0f);
        spec.transparent.setRGB(1.0f, 1.0f, 1.0f);
        spec.emissive.setRGB(0.0f, 0.0f, 0.0f);
        spec.roughness = spec.cloudiness = 0.0;
        spec.bumpGrad.set(0.0, 0.0, 0.0);
        return;
      }
    texture.getTextureSpec(spec);
  }

  public void getTransparency(Vec3 pos, RGBColor trans, double angle, double size, double t, double param[])
  {
    if (!appliesToFace(angle > 0.0))
      {
        trans.setRGB(1.0f, 1.0f, 1.0f);
        return;
      }
    texture.getTransparency(trans);
  }

  public double getDisplacement(Vec3 pos, double size, double t, double param[])
  {
    return 0.0;
  }

  public static boolean legalMapping(Object3D obj, Texture tex)
  {
    return (tex instanceof UniformTexture);
  }

  public TextureMapping duplicate()
  {
    UniformMapping map = new UniformMapping(texture);
    map.setAppliesTo(appliesTo());
    return map;
  }
  
  public TextureMapping duplicate(Texture tex)
  {
    UniformMapping map = new UniformMapping(tex);
    map.setAppliesTo(appliesTo());
    return map;
  }
  
  public void copy(TextureMapping map)
  {
    setAppliesTo(map.appliesTo());
  }

  public Panel getEditingPanel(ObjectInfo obj, final MaterialPreviewer preview)
  {
      Panel p = new Panel();
      final Choice c = new Choice();
      p.add(new Label("Apply To:"));
      p.add(c);
      c.add("Front and Back Faces");
      c.add("Front Faces Only");
      c.add("Back Faces Only");
      c.select(appliesTo());
      c.addItemListener(new ItemListener() {
        public void itemStateChanged(ItemEvent e)
        {
          setAppliesTo((short) c.getSelectedIndex());
          preview.setTexture(getTexture(), UniformMapping.this);
          preview.render();
        }
      } );
      return p;
  }
  
  public UniformMapping(DataInputStream in, Texture theTexture) throws IOException, InvalidObjectException
  {
    short version = in.readShort();
    
    if (version < 0 || version > 1)
      throw new InvalidObjectException("");
    if (version == 1)
      setAppliesTo(in.readShort());
    texture = (UniformTexture) theTexture;
  }
  
  public void writeToFile(DataOutputStream out) throws IOException
  {
    out.writeShort(1);
    out.writeShort(appliesTo());
  }
}