/* This class is attached to a scroll pane.  It causes the scroll pane to automatically
   scroll whenever the mouse is dragged beyond the edge of it. */

/* Copyright (C) 2001 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY 
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion.ui;

import java.awt.*;
import java.awt.event.*;

public class AutoScroller implements MouseListener, MouseMotionListener, Runnable
{
  protected Component target;
  protected ScrollPane sp;
  protected Thread scrollThread;
  protected int x, y, xinc, yinc, delay;
  
  public AutoScroller(Component scroll, ScrollPane pane, int xincrement, int yincrement)
  {
    xinc = xincrement;
    yinc = yincrement;
    target = scroll;
    sp = pane;
    target.addMouseListener(this);
    target.addMouseMotionListener(this);
    delay = 100;
  }
  
  public void mousePressed(MouseEvent ev)
  {
    x = ev.getX();
    y = ev.getY();
    scrollThread = new Thread(this);
    scrollThread.start();
  }

  public void mouseReleased(MouseEvent ev)
  {
    scrollThread.interrupt();
    scrollThread = null;
  }

  public void mouseClicked(MouseEvent ev)
  {
  }

  public void mouseEntered(MouseEvent ev)
  {
  }

  public void mouseExited(MouseEvent ev)
  {
  }

  public void mouseMoved(MouseEvent ev)
  {
  }
  
  public void mouseDragged(MouseEvent ev)
  {
    x = ev.getX();
    y = ev.getY();
  }

  public void run()
  {
    while (true)
      {
        Dimension scrollSize = sp.getViewportSize();
        Point scrollPos = sp.getScrollPosition();
        if (x < scrollPos.x || y < scrollPos.y || x > scrollPos.x+scrollSize.width || y > scrollPos.y+scrollSize.height)
          scrollWhileDragging(scrollSize, scrollPos);
        try
          {
            Thread.sleep(delay);
          }
        catch (InterruptedException ex)
          {
            return;
          }
      }
  }
  
  /* This is called repeatedly whenever the mouse is dragged outside the visible bounds
     to scroll the ScrollPane.  If additional things need to be done at this time (such
     redrawing objects being dragged), this can be subclassed. */

  protected void scrollWhileDragging(Dimension scrollSize, Point scrollPos)
  {
    Point newPos = scrollPos;
    if (x < scrollPos.x)
      newPos.x -= xinc;
    if (y < scrollPos.y)
      newPos.y -= yinc;
    if (x > scrollPos.x+scrollSize.width)
      newPos.x += xinc;
    if (y > scrollPos.y+scrollSize.height)
      newPos.y += yinc;
    sp.setScrollPosition(newPos);
    x += newPos.x - scrollPos.x;
    y += newPos.y - scrollPos.y;
  }
}
