/* An EntriesDialog is a modal dialog containing a line of text, and one or more TextFields
   for the user to fill out.  Each TextField has a label next to it.  At the bottom are two
   buttons labeled OK and Cancel. */
   
/* Copyright (C) 1999,2000,2002 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY 
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion.ui;

import artofillusion.*;
import java.awt.*;
import java.awt.event.*;

public class EntriesDialog extends Dialog implements ActionListener, KeyListener
{
  TextField fields[];
  String initialValues[], finalValues[];

  public EntriesDialog(Window parent, String prompt, String[] labels)
  {
    this(parent, prompt, labels, new String [labels.length]);
  }

  public EntriesDialog(Window parent, String prompt, String[] labels, String[] initialEntries)
  {
    super(Utilities.findFrame(parent), true);

    GridBagConstraints c = new GridBagConstraints();
    Panel p = new Panel();
    int i;
    
    initialValues = initialEntries;
    setLayout(new GridBagLayout());
    c.gridwidth = 2;
    add(new Label(prompt), c);
    c.anchor = GridBagConstraints.EAST;
    c.ipadx = 5;
    c.gridwidth = 1;
    c.gridx = 0;
    fields = new TextField [labels.length];
    for (i = 0; i < labels.length; i++)
      add(new Label(labels[i]), c);
    c.fill = GridBagConstraints.HORIZONTAL;
    c.ipadx = 0;
    c.gridx = 1;
    for (i = 0; i < labels.length; i++)
      {
	if (initialEntries[i] == null)
	  add(fields[i] = new TextField(), c);
	else
	  add(fields[i] = new TextField(initialEntries[i]), c);
	fields[i].addKeyListener(this);
      }
    c.gridwidth = 2;
    c.gridx = 0;
    c.gridy = labels.length+1;
    add(p, c);
    p.add(Translate.button("ok", this));
    p.add(Translate.button("cancel", this));
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e)
      {
        closeWindow(false);
      }
    } );
    pack();
    setResizable(false);
    ModellingApp.centerDialog(this, parent);
    fields[0].requestFocus();
    fields[0].selectAll();
    show();
  }
  
  /* Return the array of entries.  If the user clicked cancel, then return null. */
  
  public String [] getEntries()
  {
    return finalValues;
  }

  public Insets getInsets()
  {
    Insets in = super.getInsets();

    return new Insets(in.top+10, in.left+10, in.bottom+10, in.right+10);
  }

  public void actionPerformed(ActionEvent e)
  {
    String command = e.getActionCommand();
    int i;

    closeWindow(command.equals("ok"));
  }
    
  /* Pressing Return and Escape are equivalent to clicking OK and Cancel. */
    
  public void keyPressed(KeyEvent ev)
  {
    int i, code = ev.getKeyCode();

    if (code != KeyEvent.VK_ENTER && code != KeyEvent.VK_ESCAPE)
      return;
    closeWindow(code == KeyEvent.VK_ENTER);
  }
  
  private void closeWindow(boolean ok)
  {
    if (ok)
      {
	finalValues = new String [fields.length];
	for (int i = 0; i < fields.length; i++)
	  finalValues[i] = fields[i].getText();
      }
    else
      finalValues = null;
    dispose();
  }
  
  /* Unused KeyListener methods. */

  public void keyReleased(KeyEvent ev)
  {
  }

  public void keyTyped(KeyEvent ev)
  {
  }
}
