/* This class is a Dialog which "floats" above its parent Frame whenever it is set to be
   non-modal.  That is, it is drawn above its parent, but always makes sure that the
   parent has focus so that it will respond to key-presses. */

/* Copyright (C) 2001 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY 
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion.ui;

import java.awt.*;
import java.awt.event.*;

public class FloatingDialog extends Dialog
{
  public FloatingDialog(Frame parent)
  {
    super(parent);
    addWindowListener(new Floater());
  }

  public FloatingDialog(Frame parent, boolean modal)
  {
    super(parent, modal);
    addWindowListener(new Floater());
  }

  public FloatingDialog(Frame parent, String title)
  {
    super(parent, title);
    addWindowListener(new Floater());
  }

  public FloatingDialog(Frame parent, String title, boolean modal)
  {
    super(parent, title, modal);
    addWindowListener(new Floater());
  }
  
  private class Floater extends WindowAdapter
  {
    public void windowActivated(WindowEvent e)
    {
      if (!isModal())
        ((Window) getParent()).toFront();
    }
  }
}
