/* Copyright (C) 1999,2000,2002 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion.ui;

import artofillusion.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;


/** Description of an item in the itemList and "object" of the MenuTreeElement
*/

public class MenuDescription
{
  public boolean menu;
  public String internalName;
  public String labelText;

  public MenuDescription(String descText, boolean menu)
  {
    setDescText(descText);
    this.menu = menu;
  }

  public MenuDescription(String descText)
  {
    this(descText, false);
  }

  /** For saving a label in a *.menu file, spaces are replaced by '\s'
    and '\' is replaced by '\\'
    */
  public static String quoteLabel(String l)
  {
    StringBuffer result = new StringBuffer();

    for(int i = 0; i < l.length(); ++i)
    {
      if (l.charAt(i) == ' ')
        result.append("\\s");
      else if (l.charAt(i) == '\\')
        result.append("\\\\");
      else
        result.append(l.charAt(i));
    }
    return result.toString();
  }

  /** Opposite of {@link #quoteLabel}. */

  public static String unquoteLabel(String l)
  {
    StringBuffer result = new StringBuffer();

    for(int i = 0; i < l.length(); ++i)
    {
      if (l.charAt(i) == '\\')
      {
        i++;
        if (l.charAt(i) == 's')
          result.append(" ");
        else if (l.charAt(i) == '\\')
          result.append("\\");
      }
      else
        result.append(l.charAt(i));
    }
    return result.toString();
  }

  public void setDescText(String descText)
  {
    String[] pieces = Utilities.breakLine(descText);
    internalName = pieces[0];
    if (pieces.length > 2)
      labelText = unquoteLabel(pieces[2]);
    else
      labelText = "";
  }

  public String getDescText()
  {
    if (! "".equals(labelText))
      return internalName+" - "+quoteLabel(labelText);
    else
      return internalName;
  }

  public boolean equals(Object o)
  {
    if (!(o instanceof MenuDescription))
      return false;

    MenuDescription d = (MenuDescription)o;

    return d.internalName.equals(internalName) && (d.menu == menu);
  }

  /** Returns a string which is shown in the tree and the menu item list. */
  public String getPresentableString()
  {
    return getLabel()+" ("+internalName+")";
  }
  
  /** Returns the label of the item. If labelText is equal "",
    the default label ist returned. */
  
  public String getLabel()
  {
    if (! "".equals(labelText))
      return labelText;
    else
      return LayoutWindow.getDefaultMenuLabel(internalName);
  }
      

}

