
/* Copyright (C) 1999,2000,2002 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion.ui;

import artofillusion.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/* A MenuEditDialog is a modal dialog to edit menu structure. */

public class MenuEditDialog extends Dialog implements ActionListener,
                                            KeyListener, ItemListener
{
  boolean ok;

  private java.awt.TextField itemNameField;
  private java.awt.Choice filterChoice;
  private TreeList treeList;
  private ScrollPane treeListScroller;
  private java.awt.Choice menuChoice;
  private java.awt.Button okButton;
  private java.awt.Button transferButton;
  private java.awt.Button cancelButton;
  private java.awt.List itemList;
  LayoutWindow layoutWindow;
  
  MenuTreeElement rootElement;
  String currentMenuName;
  
  /** Vector of Desc objects. Each element corresponds with one item
    in itemList. */
  Vector itemListDescs;
  
  MenuTreeElement currentTreeEl;
  MenuDescription currentTreeElDesc;

  /** Create a non-modal dialog for editing menus

      @param parent       the parent of the dialog
      @param layoutWindow  the LayoutWindow (can be identical to parent)
  */

  public MenuEditDialog(Window parent, LayoutWindow layoutWindow)
  {
    super(Utilities.findFrame(parent), "Edit Menu", true);
    
    this.layoutWindow = layoutWindow;
    itemListDescs = new Vector();

    java.awt.GridBagConstraints gridBagConstraints;

    menuChoice = new java.awt.Choice();
    filterChoice = new java.awt.Choice();
    itemList = new java.awt.List();
    transferButton = new java.awt.Button(" ---> ");
    transferButton.setActionCommand("add");
    transferButton.addActionListener(this);
    treeList = new TreeList(layoutWindow);
    treeList.setAllowMultiple(false);
    treeList.addItemListener(this);

    treeListScroller = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
    treeListScroller.setBackground(Color.white);
    treeListScroller.add(treeList);
    treeListScroller.getVAdjustable().setUnitIncrement(10);
    new AutoScroller(treeList, treeListScroller, 0, 10);

    Label label1 = new java.awt.Label("Item name:");
    itemNameField = new java.awt.TextField();
    okButton = Translate.button("ok", this);
    cancelButton = Translate.button("cancel", this);

    setLayout(new java.awt.GridBagLayout());

//    gridBagConstraints = new java.awt.GridBagConstraints();
//    gridBagConstraints.gridx = 0;
//    gridBagConstraints.gridy = 0;
//    gridBagConstraints.gridwidth = 3;
//    gridBagConstraints.anchor = gridBagConstraints.NORTHWEST;
//    gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
//    gridBagConstraints.weightx = 1;
//    gridBagConstraints.weighty = 1;
//    add(menuChoice, gridBagConstraints);
//
//    gridBagConstraints = new java.awt.GridBagConstraints();
//    gridBagConstraints.gridx = 0;
//    gridBagConstraints.gridy = 1;
//    gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
//    //gridBagConstraints.weightx = 1;
//    //gridBagConstraints.weighty = 1;
//    add(filterChoice, gridBagConstraints);

    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 0;
    gridBagConstraints.gridy = 2;
    gridBagConstraints.gridwidth = 2;
    gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
    //gridBagConstraints.anchor = gridBagConstraints.NORTHWEST;
    //gridBagConstraints.ipadx = 50;
    gridBagConstraints.weightx = 1;
    gridBagConstraints.weighty = 1;
    add(itemList, gridBagConstraints);

    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 2;
    gridBagConstraints.gridy = 2;
    gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
    gridBagConstraints.weightx = 5;
    gridBagConstraints.weighty = 1;
    add(transferButton, gridBagConstraints);

    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 3;
    gridBagConstraints.gridy = 2;
    //gridBagConstraints.gridheight = 6;
    //gridBagConstraints.anchor = gridBagConstraints.NORTHEAST;
    gridBagConstraints.fill = GridBagConstraints.BOTH;
    gridBagConstraints.weightx = 1;
    gridBagConstraints.weighty = 1;
    add(treeListScroller, gridBagConstraints);

    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 0;
    gridBagConstraints.gridy = 3;
    gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
    gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
    //gridBagConstraints.weightx = 19;
    //gridBagConstraints.weighty = 1;
    add(label1, gridBagConstraints);

    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridy = 3;
    gridBagConstraints.gridwidth = 3;
    gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
    //gridBagConstraints.weightx = 1;
    //gridBagConstraints.weighty = 1;
    add(itemNameField, gridBagConstraints);

    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 0;
    gridBagConstraints.gridy = 4;
    gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
    gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
    //gridBagConstraints.weightx = 1;
    //gridBagConstraints.weighty = 1;
    add(okButton, gridBagConstraints);

    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridy = 4;
    gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
    gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
    //gridBagConstraints.weightx = 1;
    //gridBagConstraints.weighty = 1;
    add(cancelButton, gridBagConstraints);

    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e)
      {
        ok = false;
        closeWindow();
      }
    } );
    addAsListener(this);
    pack();
    setBounds(10,10, 400, 300);
//    setResizable(false);
    ModellingApp.centerDialog(this, parent);
    loadMenu("MainWindow");
    show();
  }

  /** Return true if the user clicked OK, false if they clicked Cancel. */

  public boolean clickedOk()
  {
    return ok;
  }

  public Insets getInsets()
  {
    Insets in = super.getInsets();

    return new Insets(in.top+10, in.left+10, in.bottom+10, in.right+10);
  }
  
  /** Fill a unique list of all items in this structure. Internal recursive version.
   */
  // TODO(MB) Faster method?
  protected static Vector fillUniqueList(Object[] itemdescs, Vector list)
  {
    //String s;
    MenuDescription d;
    
    d = (MenuDescription)itemdescs[0];
    if (list.indexOf(d) == -1)
      list.add(d);
    
    for(int i = 1; i < itemdescs.length; ++i)
    {
      if (itemdescs[i] instanceof Object[])
      {
	fillUniqueList((Object[])itemdescs[i], list);
      }
      else if (itemdescs[i] instanceof MenuDescription)
      {
        d = (MenuDescription)itemdescs[i];
        if ( list.indexOf(d) == -1 && !d.internalName.equals("root")
        && !(d.internalName.indexOf('/')>-1) )
          list.add(d);
      }
    }
    return list;
  }

  /** Loads appropriate data for a particular menu and fills it into the
    components of the dialog
  */
  public void loadMenu(String menuname)
  {
    int i;
    
    currentMenuName = menuname;

    // From the default list get all possible menu item
    
    Object[] structdata = new MenuStructure(menuname, true).getData();
    
    itemListDescs.removeAllElements();
    itemListDescs = fillUniqueList(structdata, new Vector());
    
    
    if (menuname.equals("MainWindow"))
    {  // Add Tools and Scripts
      
      // Add Modelling Tools
      ModellingTool tools[] = ModellingApp.getModellingTools();
      for(i = 0; i < tools.length; ++i)
      {
        itemListDescs.add(new MenuDescription("modellingTool/"+tools[i].getClass().getName()+
            " - "+tools[i].getName(), false));
      }
      
      // Add Scripts
      String[] scripts = layoutWindow.getScriptList();
      for(i = 0; i < scripts.length; ++i)
      {
        itemListDescs.add(new MenuDescription(scripts[i], false));
      }
      
      // Imports      
      Translator trans[] = ModellingApp.getTranslators();

      for (i = 0; i < trans.length; i++)
        if (trans[i].canImport())
          itemListDescs.add(new MenuDescription("import/"+trans[i].getClass().getName(), false));

      // Exports
      for (i = 0; i < trans.length; i++)
        if (trans[i].canExport())
          itemListDescs.add(new MenuDescription("export/"+trans[i].getClass().getName(), false));
    }    
    
    itemList.removeAll();
    for(i = 0; i < itemListDescs.size(); ++i)
      itemList.add(((MenuDescription)itemListDescs.get(i)).getPresentableString());


    // Get (possibly customized) structure and build up tree of MenuTreeItem's
    MenuStructure structure = new MenuStructure(menuname, false);
    rootElement = structure.createTreeElements(treeList);
    rootElement.setAsRoot();
    rootElement.setExpanded(true);

    treeList.removeAllElements();
    treeList.addElement(rootElement);
  }

  public void saveMenu()
  {
    if (rootElement == null)
      return;
    MenuStructure struct = new MenuStructure();
    struct.buildFromMenuTree(rootElement);
    struct.saveStructure(currentMenuName);
    
    rootElement = null;
  }
  
  /** Transfer the label entered in itemNameField into the data of the current
    tree element. */
  public void transferToCurrentTreeEl()
  {
    if (currentTreeEl == null)
      return;
    
    currentTreeElDesc.labelText = itemNameField.getText();
    currentTreeEl.rereadLabel();
    treeList.updateTree();
    currentTreeEl = null;
    currentTreeElDesc = null;
  }
      
    
  public void actionPerformed(ActionEvent e)
  {
    String command = e.getActionCommand();

    if (command.equals("cancel"))
    {
      ok = false;
      closeWindow();
    }
    else if (command.equals("ok"))
    {
      ok = true;
      closeWindow();
    }
    else if (command.equals("add"))
    {
      int idx = itemList.getSelectedIndex();
      MenuDescription d = (MenuDescription)itemListDescs.get(idx);
      if (d.menu)
      {
        rootElement.addChild(new MenuTreeElement(d,
            treeList, new MenuTreeElement[0]), 0);
        treeList.updateTree();
      }
      else
      {
        rootElement.addChild(new MenuTreeElement(d,
            treeList, null), 0);
        treeList.updateTree();
      }
    }
  }
  
  public void itemStateChanged(ItemEvent e)
  {
    if (e.getSource() == treeList)
    {
      transferToCurrentTreeEl();
      TreeElement[] selected = treeList.getSelectedElements();
      if (selected.length == 1)
      {
        currentTreeEl = (MenuTreeElement) selected[0];
        currentTreeElDesc = currentTreeEl.getItemDescription();
        itemNameField.setText(currentTreeElDesc.labelText);
      }
    }
  }

  private void closeWindow()
  {
//    if (ok && okCallback != null)
//      okCallback.execute();
//    if (!ok && cancelCallback != null)
//      cancelCallback.execute();
    transferToCurrentTreeEl();
    
    if (ok)
      saveMenu();
      
    dispose();
    removeAsListener(this);
  }

  
  /* Pressing Return and Escape are equivalent to clicking OK and Cancel. */

  public void keyPressed(KeyEvent ev)
  {
    int code = ev.getKeyCode();

    if ( (code == KeyEvent.VK_BACK_SPACE || code == KeyEvent.VK_DELETE) &&
         ev.getSource() == treeList)
    {
      Object[] todelete = treeList.getSelectedObjects();
      TreeElement[] els = treeList.getSelectedElements();

      for(int i = 0; i < todelete.length; ++i)
        if (((MenuTreeElement)els[i]).isDeletable())
          treeList.removeObject(todelete[i]);
        
      return;
    }
    else if (code == KeyEvent.VK_ENTER && ev.getSource() instanceof Button)
      return;
    else if (code == KeyEvent.VK_ENTER || code == KeyEvent.VK_ESCAPE)
    {
      ok = (code == KeyEvent.VK_ENTER);
      closeWindow();
      return;
    }  
  }

  /* Unused KeyListener methods. */

  public void keyReleased(KeyEvent ev)
  {
  }

  public void keyTyped(KeyEvent ev)
  {
  }

  // TODO(MB) Factor out

  /** Add this as a key listener to every component. */

  private void addAsListener(Component c)
  {
    c.addKeyListener(this);
    if (c instanceof Container)
      {
	Component children[] = ((Container) c).getComponents();
	for (int i = 0; i < children.length; i++)
	  addAsListener(children[i]);
      }
  }

  /** Remove this as a key listener before returning. */

  private void removeAsListener(Component c)
  {
    c.removeKeyListener(this);
    if (c instanceof Container)
      {
	Component children[] = ((Container) c).getComponents();
	for (int i = 0; i < children.length; i++)
	  removeAsListener(children[i]);
      }
  }
}
