/* Copyright (C) 2002 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY 
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion.ui;

import java.awt.*;

/** This interface represents an object which creates single items of a menu.
 
   @author Michael Butscher
 */

public interface MenuItemFactory
{
  
  /** Creates a MenuItem which is either a normal MenuItem,
    a CheckboxMenuItem or a whole submenu, depending on
    {@link #isMenuItemCheckbox}.
    A appropriate listener is registered for this automatically.
    
    @param name Internal name of the item (as recognized by action listener)
    @param label Label of the item (or null for default label)
    @param shortcut Shortcut of the item (or null for default)
    
    @return The menu item or null if creation is not allowed.
  */

  public MenuItem createMenuItem(String name, MenuShortcut shortcut,
      String label);

}