/* A MessageDialog is a modal dialog containing one or more lines of text, and one or more 
   buttons for the user to select from.  Clicking any of the buttons dismisses the dialog. */

/* Copyright (C) 1999,2000,2002 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY 
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion.ui;

import artofillusion.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class MessageDialog extends Dialog implements KeyListener
{
  int pressed, i, number;

  public MessageDialog(Window parent, String message)
  {
    this(parent, breakString(message));
  }

  public MessageDialog(Window parent, String message, String[] choices)
  {
    this(parent, breakString(message), choices);
  }

  public MessageDialog(Window parent, String[] message)
  {
    this(parent, message, new String[] {Translate.text("button.ok")});
  }

  public MessageDialog(Window parent, String[] message, String[] choices)
  {
    super(Utilities.findFrame(parent), true);

    Button btn;
    Panel buttons = new Panel();
    
    add(buttons, "South");
    for (i = 0; i < choices.length; i++)
      {
	btn = new Button(choices[i]);
	btn.addActionListener(new ActionListener() {
	  int number = i;
	  public void actionPerformed(ActionEvent e) {pressed = number; dispose();} });
	buttons.add(btn);
      }
    Panel msg = new Panel() {
      public Insets getInsets()
	{
	  Insets in = super.getInsets();
	  return new Insets(in.top+10, in.left+10, in.bottom+10, in.right+10);
	} };
    msg.setLayout(new GridBagLayout());
    add(msg, "Center");
    GridBagConstraints gc = new GridBagConstraints();
    gc.gridx = 0;
    for (i = 0; i < message.length; i++)
      msg.add(new Label(message[i]), gc);
    number = choices.length;
    addAsListener(this);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e)
      {
        pressed = number-1;
        dispose();
      }
    } );
    pack();
    setResizable(false);
    ModellingApp.centerDialog(this, parent);
    show();
  }
  
  /** Break a string into lines. */
  
  private static String [] breakString(String s)
  {
    int lines = (s.length()/60)+1;
    if (lines < 2)
      return new String [] {s};
    int lineLength = s.length()/lines;
    Vector line = new Vector();
    int index = 0;
    while (index+lineLength < s.length())
      {
        int next = s.indexOf(' ', index+lineLength);
        if (next == -1)
          next = s.length();
        line.addElement(s.substring(index, next).trim());
        index = next;
      }
    if (index < s.length())
      line.addElement(s.substring(index).trim());
    String result[] = new String [line.size()];
    line.copyInto(result);
    return result;
  }
  
  /** Get the index of the button that was pressed. */
  
  public int getChoice()
  {
    return pressed;
  }

  /* Add this as a key listener to every component. */
  
  private void addAsListener(Component c)
  {
    c.addKeyListener(this);
    if (c instanceof Container)
      {
	Component children[] = ((Container) c).getComponents();
	for (int i = 0; i < children.length; i++)
	  addAsListener(children[i]);
      }
  }
    
  /* Pressing Escape is equivalent to clicking the last button. */
    
  public void keyPressed(KeyEvent ev)
  {
    if (ev.getKeyCode() == KeyEvent.VK_ESCAPE)
      pressed = number-1;
    else if (ev.getKeyCode() == KeyEvent.VK_ENTER)
      pressed = 0;
    else
      return;
    dispose();
  }
  
  /* Unused KeyListener methods. */

  public void keyReleased(KeyEvent ev)
  {
  }

  public void keyTyped(KeyEvent ev)
  {
  }
}

