/* A PanelDialog is a modal dialog containing a line of text at the top, and a Panel
   which can contain any set of components.  At the bottom are two buttons labeled OK 
   and Cancel. */
   
/* Copyright (C) 2000,2002 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY 
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion.ui;

import artofillusion.*;
import java.awt.*;
import java.awt.event.*;

public class PanelDialog extends Dialog implements ActionListener, KeyListener
{
  boolean ok;

  /** Create a modal dialog containing a panel.
  
      @param parent       the parent of the dialog
      @param prompt       a text string to appear at the top of the dialog
      @param thePanel     the panel to display
  */

  public PanelDialog(Window parent, String prompt, Panel thePanel)
  {
    super(Utilities.findFrame(parent), true);

    GridBagConstraints c = new GridBagConstraints();
    Panel p;
    int i;
    
    setLayout(new GridBagLayout());
    c.gridx = 0;
    add(new Label(prompt), c);
    add(thePanel, c);
    add(p = new Panel(), c);
    p.add(Translate.button("ok", this));
    p.add(Translate.button("cancel", this));
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e)
      {
        ok = false;
        dispose();
        removeAsListener(PanelDialog.this);
      }
    } );
    addAsListener(this);
    pack();
    setResizable(false);
    ModellingApp.centerDialog(this, parent);
    show();
  }
  
  /** Return true if the user clicked OK, false if they clicked Cancel. */
  
  public boolean clickedOk()
  {
    return ok;
  }

  public Insets getInsets()
  {
    Insets in = super.getInsets();

    return new Insets(in.top+10, in.left+10, in.bottom+10, in.right+10);
  }

  public void actionPerformed(ActionEvent e)
  {
    String command = e.getActionCommand();

    if (command.equals("cancel"))
      ok = false;
    else
      ok = true;
    dispose();
    removeAsListener(this);
  }
    
  /* Pressing Return and Escape are equivalent to clicking OK and Cancel. */
    
  public void keyPressed(KeyEvent ev)
  {
    int code = ev.getKeyCode();

    if (code != KeyEvent.VK_ENTER && code != KeyEvent.VK_ESCAPE)
      return;
    if (code == KeyEvent.VK_ENTER && ev.getSource() instanceof Button)
      return;
    ok = (code == KeyEvent.VK_ENTER);
    dispose();
    removeAsListener(this);
  }
  
  /* Unused KeyListener methods. */

  public void keyReleased(KeyEvent ev)
  {
  }

  public void keyTyped(KeyEvent ev)
  {
  }

  /* Add this as a key listener to every component. */
  
  private void addAsListener(Component c)
  {
    c.addKeyListener(this);
    if (c instanceof Container)
      {
	Component children[] = ((Container) c).getComponents();
	for (int i = 0; i < children.length; i++)
	  addAsListener(children[i]);
      }
  }
  
  /* Remove this as a key listener before returning. */
  
  private void removeAsListener(Component c)
  {
    c.removeKeyListener(this);
    if (c instanceof Container)
      {
	Component children[] = ((Container) c).getComponents();
	for (int i = 0; i < children.length; i++)
	  removeAsListener(children[i]);
      }
  }
}
