/* This dialog box allows the user to specify options for creating a tube. */

/* Copyright (C) 2002 by Peter Eastman

   This program is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT ANY 
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE.  See the GNU General Public License for more details. */

package artofillusion.tools;

import artofillusion.*;
import artofillusion.math.*;
import artofillusion.object.*;
import artofillusion.ui.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public class TubeDialog extends Dialog implements ActionListener, ItemListener, TextListener
{
  LayoutWindow window;
  Curve theCurve;
  ObjectInfo curveInfo;
  Tube theTube;
  Button okButton, cancelButton;
  ValueField thicknessField;
  Choice endsChoice;
  ObjectPreviewCanvas preview;
  
  private static int counter = 1;

  public TubeDialog(LayoutWindow window, ObjectInfo curve)
  {
    super(window, "Tube", true);
    this.window = window;
    curveInfo = curve;
    theCurve = (Curve) curve.object;
    Scene scene = window.getScene();
    
    // Layout the window.
    
    setLayout(new GridBagLayout());
    GridBagConstraints gc = new GridBagConstraints();
    gc.gridx = 0;
    add(new Label("Tube Width"), gc);
    add(new Label("Cap Ends"), gc);
    gc.gridx = 1;
    gc.fill = gc.HORIZONTAL;
    add(thicknessField = new ValueField(0.1, ValueField.POSITIVE, 5), gc);
    thicknessField.addTextListener(this);
    add(endsChoice = new Choice(), gc);
    endsChoice.add("Open Ends");
    endsChoice.add("Flat Ends");
    endsChoice.setEnabled(!theCurve.isClosed());
    endsChoice.addItemListener(this);

    // Add the preview canvas.
    
    gc.gridx = 0;
    gc.gridwidth = 2;
    gc.fill = gc.BOTH;
    gc.weightx = gc.weighty = 1.0;
    add(preview = new ObjectPreviewCanvas(null), gc);
    preview.setPreferredSize(150, 150);
    
    // Add the buttons at the bottom.
    
    gc.gridx = 0;
    gc.gridwidth = 2;
    gc.weightx = gc.weighty = 0.0;
    Panel p = new Panel();
    p.add(okButton = Translate.button("ok", this));
    p.add(cancelButton = Translate.button("cancel", this));
    add(p, gc);
    makeObject();
    pack();
    show();
  }

  /** Rebuild the object when the desired thickness is changed. */

  public void textValueChanged(TextEvent e)
  {
    makeObject();
  }

  
  /** Deal with changes to the choice. */
  
  public void itemStateChanged(ItemEvent e)
  {
    makeObject();
  }
  
  // Deal with clicks on the buttons.
  
  public void actionPerformed(ActionEvent e)
  {
    if (okButton == e.getSource())
      {
	window.addObject(theTube, curveInfo.coords.duplicate(), "Tube "+(counter++), null);
	window.setSelection(window.getScene().getNumObjects()-1);
	window.setUndoRecord(new UndoRecord(window, false, UndoRecord.DELETE_OBJECT, new Object [] {new Integer(window.getScene().getNumObjects()-1)}));
	window.updateImage();
	dispose();
      }
    else
      dispose();
  }
  
  // Create the Tube.
  
  private void makeObject()
  {
    MeshVertex vert[] = theCurve.getVertices();
    double thickness[] = new double [vert.length];
    for (int i = 0; i < thickness.length; i++)
      thickness[i] = thicknessField.getValue();
    int endsStyle;
    if (theCurve.isClosed())
      endsStyle = Tube.CLOSED_ENDS;
    else if (endsChoice.getSelectedIndex() == 0)
      endsStyle = Tube.OPEN_ENDS;
    else
      endsStyle = Tube.FLAT_ENDS;
    theTube = new Tube(theCurve, thickness, endsStyle);
    theTube.setTexture(window.getScene().getDefaultTexture());
    theTube.setTextureMapping(theTube.getTexture().getDefaultMapping());
    preview.setObject(theTube);
    preview.updateImage();
    preview.repaint();
  }
}